#include "stdhdrs.h"
#include <iostream>
#include "NetComm.h"
#include "Connection.h"

using namespace std;

enum Role
{
	unknownRole,
	serverRole,
	sendOpenClientRole,
	rawClientRole
};

struct CmdArgs
{
	CmdArgs(NetComm *inNet)
		: role(unknownRole), port(NULL), server(NULL), data(NULL), 
		parseError(false), net(inNet) { }

	Role role;
	wchar_t *port;
	wchar_t *server;
	wchar_t *data;
	bool parseError;
	NetComm *net;
};

void ParseCmdArgs(int argc, wchar_t *argv[], CmdArgs *cmdArgs)
{
	for (int argIdx = 1; argIdx < argc; ++argIdx)
	{
		if (!wcscmp(argv[argIdx], L"--role"))
		{
			if (argIdx + 1 < argc)
			{
				++argIdx;
				if (!wcscmp(argv[argIdx], L"server"))
				{
					cmdArgs->role = serverRole;
				}
				else if (!wcscmp(argv[argIdx], L"raw"))
				{
					cmdArgs->role = rawClientRole;
				}
				else if (!wcscmp(argv[argIdx], L"sendOpen"))
				{
					cmdArgs->role = sendOpenClientRole;
				}
				else
				{
					cmdArgs->parseError = true;
				}
			}
			else
			{
				cmdArgs->parseError = true;
			}
		}
		else if (!wcscmp(argv[argIdx], L"--acceptClient"))
		{
			if (argIdx + 1 < argc)
			{
				cmdArgs->net->AddAcceptableClient(argv[++argIdx]);
			}
			else
			{
				cmdArgs->parseError = true;
			}
		}
		else if (!wcscmp(argv[argIdx], L"--port"))
		{
			if (argIdx + 1 < argc)
			{
				cmdArgs->port = argv[++argIdx];
			}
			else
			{
				cmdArgs->parseError = true;
			}
		}
		else if (!wcscmp(argv[argIdx], L"--server"))
		{
			if (argIdx + 1 < argc)
			{
				cmdArgs->server = argv[++argIdx];
			}
			else
			{
				cmdArgs->parseError = true;
			}
		}
		else if (!wcscmp(argv[argIdx], L"--file") ||
			!wcscmp(argv[argIdx], L"--msg"))
		{
			if (argIdx + 1 < argc)
			{
				cmdArgs->data = argv[++argIdx];
			}
			else
			{
				cmdArgs->parseError = true;
			}
		}
	}
}

void ApplyCmdArgs(CmdArgs *cmdArgs)
{

	if (!cmdArgs->parseError)
	{
		switch (cmdArgs->role)
		{
		case serverRole:
			cmdArgs->net->ListenForConnections(cmdArgs->port);
			break;

		case rawClientRole:
			if (!cmdArgs->server || !cmdArgs->data)
			{
				cmdArgs->parseError = true;
			}
			else
			{
				Connection *conn = cmdArgs->net->ConnectTo(cmdArgs->server, cmdArgs->port);
				if (conn)
				{
					conn->SendString(cmdArgs->data);
					delete conn;
				}
				else
				{
					wcerr << L"Unexpected error.  No Connection object." << endl;
				}
			}
			break;

		case sendOpenClientRole:
			if (!cmdArgs->server || !cmdArgs->data)
			{
				cmdArgs->parseError = true;
			}
			else
			{
				Connection *conn = cmdArgs->net->ConnectTo(cmdArgs->server, cmdArgs->port);
				if (conn)
				{
					wstring msg = L"open ";
					wchar_t normalPath[MAX_PATH];
					if (GetFullPathName(cmdArgs->data, ARRAYSIZE(normalPath), normalPath, NULL))
					{
						bool msgIsGood = true;
						static const wchar_t sSharePrefix[] = L"\\\\";
						if (!wcsncmp(sSharePrefix, normalPath, ARRAYSIZE(sSharePrefix) - 1))
						{
							msg += normalPath;
						}
						else
						{
							char localName[256];
							wchar_t localNameW[256];
							if (!gethostname(localName, ARRAYSIZE(localName)))
							{
								if (MultiByteToWideChar(
									CP_ACP, 0, 
									localName, strlen(localName) + 1,
									localNameW, ARRAYSIZE(localNameW)))
								{
									msg += L"\\\\";
									msg += localNameW;
									msg += L"\\";
									msg += normalPath[0];
									msg += L"$";
									msg += (normalPath + 2);
								}
								else
								{
									wcerr << L"MultiByteToWideChar error: 0x" << hex << GetLastError() << endl;
									msgIsGood = false;
								}
							}
							else
							{
								wcerr << L"gethostname error: 0x" << hex << WSAGetLastError() << endl;
								msgIsGood = false;
							}
						}

						if (msgIsGood)
						{
							conn->SendString(msg.c_str());
						}
					}
					else
					{
						wcerr << L"GetFullPathName error: 0x" << hex << GetLastError();
					}
					delete conn;
				}
				else
				{
					wcerr << L"Unexpected error.  No Connection object." << endl;
				}
			}
			break;

		default:
			cmdArgs->parseError = true;
			break;
		}
	}

	if (cmdArgs->parseError)
	{
		wcerr
			<< L"Incorrect parameter usage." << endl
			<< endl
			<< L"General instructions:" << endl
			<< L"On host named serverName run " << endl
			<< L"\tremoteopen --role server --acceptClient clientName1 --acceptClient clientName2" << endl
			<< L"On the two clients clientName1 and clientName2 run " << endl
			<< L"\tremoteopen --role sendOpen --server serverName --file C:\\users\\jane\\readme.txt" << endl
			<< L"to have the specified file on the client machine opened via UNC on the server machine." << endl
			<< endl
			<< L"Usage:" << endl
			<< L"\tremoteopen --role server [--port (portNumber)] [--acceptClient (clientHostName)]*" << endl
			<< L"\tremoteopen --role raw --server (serverName) [--port (portNumber)] --msg (raw message)" << endl
			<< L"\tremoteopen --role sendOpen --server (serverName) [--port (portNumber)] --file (fileName)" << endl
			<< endl
			<< L"If port is not specified it defaults to 8082." << endl;
	}
}

int wmain(int argc, wchar_t *argv[])
{
	NetComm net;
	CmdArgs cmdArgs(&net);
	cmdArgs.port = L"8082";

	ParseCmdArgs(argc, argv, &cmdArgs);
	ApplyCmdArgs(&cmdArgs);

	return 0;
}